﻿using CoreLibrary.Joueurs;
using System.Runtime.Serialization;

namespace CoreLibrary.Exceptions
{
    /// <summary>
    /// Exception levée lorsqu'un joueur se connecte alors qu'il l'est déjà.
    /// </summary>
    [Serializable]
    public class JoueurDejaConnecteException : Exception
    {
        // Message par défaut
        private const string messageDefaut = "Le joueur est déjà connecté.";

        /// <summary>
        /// Initialise une nouvelle instance de la classe <see cref="JoueurDejaConnecteException"/> avec le message par défaut.
        /// </summary>
        public JoueurDejaConnecteException() : base(messageDefaut)
        {}

        /// <summary>
        /// Initialise une nouvelle instance de la classe <see cref="JoueurDejaConnecteException"/> avec le message spécifié.
        /// </summary>
        public JoueurDejaConnecteException(string message) : base(message)
        {}

        /// <summary>
        /// Initialise une nouvelle instance de la classe <see cref="JoueurDejaConnecteException"/> avec le joueur spécifié.
        /// </summary>
        public JoueurDejaConnecteException(Joueur joueur) : base($"Le joueur {joueur.Nom} est déjà connecté.")
        { }

        /// <summary>
        /// Initialise une nouvelle instance de la classe <see cref="JoueurDejaConnecteException"/> avec le message et l'exception parent spécifiés.
        /// </summary>
        public JoueurDejaConnecteException(string message, Exception exception) : base(message, exception)
        {}

        [Obsolete("This method is obsolete. Use alternative methods for data retrieval.", DiagnosticId = "SYSLIB0051")]
        protected JoueurDejaConnecteException(SerializationInfo info, StreamingContext contexte) : base(info, contexte) 
        {}

        [Obsolete("This method is obsolete. Use alternative methods for data retrieval.", DiagnosticId = "SYSLIB0051")]
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
        }
    }
}
